@@ -1,5 +1,6 @@
 import ast
 import numpy as np
+import pickle
 
 from collections import Counter
 from sklearn.base import BaseEstimator, ClassifierMixin
@@ -9,6 +10,7 @@ from tensorflow.keras.preprocessing.sequence import pad_sequences
 from tensorflow.keras.optimizers import Adam
 from tensorflow.keras.callbacks import TensorBoard
 from transformers import CamembertTokenizer, XLMTokenizer
+from transformers import TFCamembertModel, TFFlaubertModel
 
 from melusine.config.config import ConfigJsonReader
 from melusine.nlp_tools.tokenizer import Tokenizer
@@ -56,6 +58,10 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         Loss function for training.
         Default value, 'categorical_crossentropy'.
 
+    activation : str, optional
+        Activation function.
+        Default value, 'softmax'.
+
     batch_size : int, optional
         Size of batches for the training of the neural network model.
         Default value, 4096.
@@ -71,6 +77,7 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
 
     bert_model : str, optional
         Model name from HuggingFace library or path to local model
+
         Only Camembert and Flaubert supported
         Default value, 'camembert-base'
 
@@ -99,57 +106,90 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
 
     """
 
-    def __init__(self,
-                 pretrained_embedding=None,
-                 architecture_function=None,
-                 text_input_column='clean_text',
-                 meta_input_list=['extension', 'dayofweek', 'hour', 'min'],
-                 vocab_size=25000,
-                 seq_size=100,
-                 embedding_dim=200,
-                 loss='categorical_crossentropy',
-                 batch_size=4096,
-                 n_epochs=15,
-                 bert_tokenizer='camembert-base',
-                 bert_model='camembert-base',
-                 **kwargs):
+    def __init__(
+        self,
+        pretrained_embedding=None,
+        architecture_function=None,
+        text_input_column="clean_text",
+        meta_input_list=["extension", "dayofweek", "hour", "min"],
+        vocab_size=25000,
+        seq_size=100,
+        embedding_dim=200,
+        loss="categorical_crossentropy",
+        activation="softmax",
+        batch_size=4096,
+        n_epochs=15,
+        bert_tokenizer="jplu/tf-camembert-base",
+        bert_model="jplu/tf-camembert-base",
+        **kwargs,
+    ):
         self.architecture_function = architecture_function
         self.pretrained_embedding = pretrained_embedding
-        if self.architecture_function.__name__ != 'bert_model':
+        if self.architecture_function.__name__ != "bert_model":
             self.tokenizer = Tokenizer(input_column=text_input_column)
         elif "camembert" in bert_tokenizer.lower():
             self.tokenizer = CamembertTokenizer.from_pretrained(bert_tokenizer)
         elif "flaubert" in bert_tokenizer.lower():
             self.tokenizer = XLMTokenizer.from_pretrained(bert_tokenizer)
         else:
-            raise NotImplementedError('Bert tokenizer {} not implemented'.format(bert_tokenizer))
+            raise NotImplementedError(
+                "Bert tokenizer {} not implemented".format(bert_tokenizer)
+            )
         self.text_input_column = text_input_column
         self.meta_input_list = meta_input_list
         self.vocab_size = vocab_size
         self.seq_size = seq_size
         self.embedding_dim = embedding_dim
         self.loss = loss
+        self.activation = activation
         self.batch_size = batch_size
         self.n_epochs = n_epochs
         self.bert_model = bert_model
+        self.nb_labels = 0
+        self.nb_meta_features = 0
+        self.vocabulary = []
+        self.vocabulary_dict = {}
 
     def save_nn_model(self, filepath):
         """Save model to pickle, json and save weights to .h5."""
-        json_model = self.model.to_json()
-        open(filepath + ".json", 'w').write(json_model)
-        self.model.save_weights(filepath + "_model_weights.h5", overwrite=True)
+
+        if self.architecture_function.__name__ != "bert_model":
+            json_model = self.model.to_json()
+            open(filepath + ".json", "w").write(json_model)
+            self.model.save_weights(filepath + "_model_weights.h5", overwrite=True)
+        else:
+            with open(filepath + "bert_params.pkl", "wb") as f:
+                pickle.dump([self.nb_labels, self.nb_meta_features], f)
+            self.model.save_weights(filepath + "_model_weights", overwrite=True)
         pass
 
     def load_nn_model(self, filepath):
         """Save model from json and load weights from .h5."""
-        model = model_from_json(open(filepath + ".json").read(),
-                                custom_objects={'PositionalEncoding': PositionalEncoding,
-                                                'TransformerEncoderLayer': TransformerEncoderLayer,
-                                                'MultiHeadAttention': MultiHeadAttention})
-        model.load_weights(filepath + "_model_weights.h5")
-        model.compile(optimizer=Adam(),
-                      loss='categorical_crossentropy',
-                      metrics=['accuracy'])
+
+        if self.architecture_function.__name__ != "bert_model":
+            model = model_from_json(
+                open(filepath + ".json").read(),
+                custom_objects={
+                    "PositionalEncoding": PositionalEncoding,
+                    "TransformerEncoderLayer": TransformerEncoderLayer,
+                    "MultiHeadAttention": MultiHeadAttention,
+                },
+            )
+            model.load_weights(filepath + "_model_weights.h5")
+            model.compile(optimizer=Adam(), loss=self.loss, metrics=["accuracy"])
+        else:
+            with open(filepath + "bert_params.pkl", "rb") as f:
+                nb_labels, nb_meta_features = pickle.load(f)
+            model = self.architecture_function(
+                ntargets=nb_labels,
+                seq_max=self.seq_size,
+                nb_meta=nb_meta_features,
+                loss=self.loss,
+                activation=self.activation,
+                bert_model=self.bert_model,
+            )
+            model.load_weights(filepath + "_model_weights")
+
         self.model = model
         pass
 
@@ -157,7 +197,7 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         """Method called before serialization for a specific treatment to save
         model weight and structure instead of standard serialization."""
         dict_attr = dict(self.__dict__)
-        if 'model' in dict_attr:
+        if "model" in dict_attr:
             del dict_attr["model"]
             del dict_attr["embedding_matrix"]
             del dict_attr["pretrained_embedding"]
@@ -168,7 +208,9 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         model weight and structure instead of standard serialization."""
         self.__dict__ = dict_attr
 
-    def fit(self, X, y, tensorboard_log_dir=None, **kwargs):
+    def fit(
+        self, X_train, y_train, tensorboard_log_dir=None, validation_data=None, **kwargs
+    ):
         """Fit the neural network model on X and y.
         If meta_input list is empty list or None the model is used
         without metadata.
@@ -177,98 +219,110 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
 
         Parameters
         ----------
-        X : pd.DataFrame
+        X_train : pd.DataFrame
 
-        y : pd.Series
+        y_train : pd.Series
 
         tensorboard_log_dir : str
             If not None, will be used as path to write logs for tensorboard
             Tensordboard callback parameters can be changed in config file
 
+        validation_data: tuple
+            Tuple of validation data
+            Data on which to evaluate the loss and any model metrics at the end of each epoch. The model will not be
+            trained on this data. This could be a tuple (x_val, y_val). validation_data will override validation_split.
+            Default value, None.
+
         Returns
         -------
         self : object
             Returns the instance
         """
+        X_input_train, y_categorical_train = self._prepare_data(X_train, y_train)
 
-        y_categorical = to_categorical(y)
-        nb_labels = len(np.unique(y))
+        if validation_data:
+            # Init X_val, y_val
+            X_val, y_val = None, None
 
-        if self.architecture_function.__name__ != 'bert_model':
-            X = self.tokenizer.transform(X)
-            if self.pretrained_embedding:
-                self._get_embedding_matrix()
-            else:
-                self._create_vocabulary_from_tokens(X)
-                self._generate_random_embedding_matrix()
-            self.vocabulary_dict = {word: i for i, word in enumerate(self.vocabulary)}
-            X_seq = self._prepare_sequences(X)
-            X_meta, nb_meta_features = self._get_meta(X)
-            self.model = self.architecture_function(
-                embedding_matrix_init=self.embedding_matrix,
-                ntargets=nb_labels,
-                seq_max=self.seq_size,
-                nb_meta=nb_meta_features,
-                loss=self.loss)
+            try:
+                X_val, y_val = validation_data
+            except Exception as e:
+                validation_data = None
+                print(
+                    "Validation_data has unexpected format. validation_data is now set to None. Following error:"
+                    + str(e)
+                )
 
-            if nb_meta_features == 0:
-                X_input = X_seq
-            else:
-                X_input = [X_seq, X_meta]
-
-        else:
-            X_seq, X_attention = self._prepare_bert_sequences(X)
-            X_meta, nb_meta_features = self._get_meta(X)
-            self.model = self.architecture_function(
-                ntargets=nb_labels,
-                seq_max=self.seq_size,
-                nb_meta=nb_meta_features,
-                loss=self.loss,
-                bert_model=self.bert_model)
-
-            if nb_meta_features == 0:
-                X_input = [X_seq, X_attention]
-            else:
-                X_input = [X_seq, X_attention, X_meta]
+            if validation_data:
+                X_input_val, y_categorical_val = self._prepare_data(
+                    X_val, y_val, validation_data=validation_data
+                )
+                validation_data = (X_input_val, y_categorical_val)
 
         if tensorboard_log_dir is None:
-            self.model.fit(X_input,
-                           y_categorical,
-                           batch_size=self.batch_size,
-                           epochs=self.n_epochs,
-                           **kwargs)
+            self.model.fit(
+                X_input_train,
+                y_categorical_train,
+                batch_size=self.batch_size,
+                epochs=self.n_epochs,
+                validation_data=validation_data,
+                **kwargs,
+            )
         else:
 
-            histogram_freq = ast.literal_eval(tensorboard_callback_parameters["histogram_freq"])
-            write_graph = ast.literal_eval(tensorboard_callback_parameters["write_graph"])
-            write_grads = ast.literal_eval(tensorboard_callback_parameters["write_grads"])
-            write_images = ast.literal_eval(tensorboard_callback_parameters["write_images"])
-            embeddings_freq = ast.literal_eval(tensorboard_callback_parameters["embeddings_freq"])
-            embeddings_layer_names = ast.literal_eval(tensorboard_callback_parameters["embeddings_layer_names"])
-            embeddings_metadata = ast.literal_eval(tensorboard_callback_parameters["embeddings_metadata"])
-            embeddings_data = ast.literal_eval(tensorboard_callback_parameters["embeddings_data"])
+            histogram_freq = ast.literal_eval(
+                tensorboard_callback_parameters["histogram_freq"]
+            )
+            write_graph = ast.literal_eval(
+                tensorboard_callback_parameters["write_graph"]
+            )
+            write_grads = ast.literal_eval(
+                tensorboard_callback_parameters["write_grads"]
+            )
+            write_images = ast.literal_eval(
+                tensorboard_callback_parameters["write_images"]
+            )
+            embeddings_freq = ast.literal_eval(
+                tensorboard_callback_parameters["embeddings_freq"]
+            )
+            embeddings_layer_names = ast.literal_eval(
+                tensorboard_callback_parameters["embeddings_layer_names"]
+            )
+            embeddings_metadata = ast.literal_eval(
+                tensorboard_callback_parameters["embeddings_metadata"]
+            )
+            embeddings_data = ast.literal_eval(
+                tensorboard_callback_parameters["embeddings_data"]
+            )
 
-            if tensorboard_callback_parameters["update_freq"] in ['batch', 'epoch']:
+            if tensorboard_callback_parameters["update_freq"] in ["batch", "epoch"]:
                 update_freq = tensorboard_callback_parameters["update_freq"]
             else:
-                update_freq = ast.literal_eval(tensorboard_callback_parameters["update_freq"])
+                update_freq = ast.literal_eval(
+                    tensorboard_callback_parameters["update_freq"]
+                )
 
-            tensorboard_callback = TensorBoard(log_dir=tensorboard_log_dir,
-                                               histogram_freq=histogram_freq,
-                                               write_graph=write_graph,
-                                               write_grads=write_grads,
-                                               write_images=write_images,
-                                               embeddings_freq=embeddings_freq,
-                                               embeddings_layer_names=embeddings_layer_names,
-                                               embeddings_metadata=embeddings_metadata,
-                                               embeddings_data=embeddings_data,
-                                               update_freq=update_freq)
-            self.model.fit(X_input,
-                           y_categorical,
-                           batch_size=self.batch_size,
-                           epochs=self.n_epochs,
-                           callbacks=[tensorboard_callback],
-                           **kwargs)
+            tensorboard_callback = TensorBoard(
+                log_dir=tensorboard_log_dir,
+                histogram_freq=histogram_freq,
+                write_graph=write_graph,
+                write_grads=write_grads,
+                write_images=write_images,
+                embeddings_freq=embeddings_freq,
+                embeddings_layer_names=embeddings_layer_names,
+                embeddings_metadata=embeddings_metadata,
+                embeddings_data=embeddings_data,
+                update_freq=update_freq,
+            )
+            self.model.fit(
+                X_input_train,
+                y_categorical_train,
+                batch_size=self.batch_size,
+                epochs=self.n_epochs,
+                callbacks=[tensorboard_callback],
+                validation_data=validation_data,
+                **kwargs,
+            )
         pass
 
     def predict(self, X, **kwargs):
@@ -298,7 +352,7 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         np.array
         """
 
-        if self.architecture_function.__name__ != 'bert_model':
+        if self.architecture_function.__name__ != "bert_model":
             X = self.tokenizer.transform(X)
             X_seq = self._prepare_sequences(X)
             X_meta, nb_meta_features = self._get_meta(X)
@@ -317,7 +371,7 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
 
     def _create_vocabulary_from_tokens(self, X):
         """Create a word indexes dictionary from tokens."""
-        token_series = X['tokens']
+        token_series = X["tokens"]
         c = Counter([token for token_list in token_series for token in token_list])
         self.vocabulary = [t[0] for t in c.most_common(self.vocab_size)]
         pass
@@ -333,12 +387,14 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         vector_dim = pretrained_embedding.embedding.vector_size
         embedding_matrix = np.zeros((vocab_size + 2, vector_dim))
         for index, word in enumerate(self.vocabulary):
-            if word not in ['PAD', 'UNK']:
-                embedding_matrix[index + 2, :] = pretrained_embedding.embedding.wv.get_vector(word)
+            if word not in ["PAD", "UNK"]:
+                embedding_matrix[
+                    index + 2, :
+                ] = pretrained_embedding.embedding.wv.get_vector(word)
         embedding_matrix[1, :] = np.mean(embedding_matrix, axis=0)
 
-        self.vocabulary.insert(0, 'PAD')
-        self.vocabulary.insert(1, 'UNK')
+        self.vocabulary.insert(0, "PAD")
+        self.vocabulary.insert(1, "UNK")
 
         self.embedding_matrix = embedding_matrix
         pass
@@ -350,10 +406,12 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         """
         vocab_size = len(self.vocabulary)
         vector_dim = self.embedding_dim
-        embedding_matrix = np.random.uniform(low=-1, high=1, size=(vocab_size + 2, vector_dim))
+        embedding_matrix = np.random.uniform(
+            low=-1, high=1, size=(vocab_size + 2, vector_dim)
+        )
         embedding_matrix[0:2, :] = np.zeros((2, vector_dim))
-        self.vocabulary.insert(0, 'PAD')
-        self.vocabulary.insert(1, 'UNK')
+        self.vocabulary.insert(0, "PAD")
+        self.vocabulary.insert(1, "UNK")
 
         self.embedding_matrix = embedding_matrix
         pass
@@ -373,9 +431,9 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         one used for the pre-trained embedding."""
 
         if isinstance(X, dict):
-            seqs = [self.tokens_to_indices(X['tokens'])]
+            seqs = [self.tokens_to_indices(X["tokens"])]
         else:
-            seqs = X['tokens'].apply(self.tokens_to_indices)
+            seqs = X["tokens"].apply(self.tokens_to_indices)
 
         X_seq = pad_sequences(seqs, maxlen=self.seq_size)
         return X_seq
@@ -387,14 +445,95 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
         """
 
         if isinstance(X, dict):
-            sequence = X['clean_text']
+            sequence = X[self.text_input_column]
         else:
-            sequence = X['clean_text'].values.tolist()
-        seqs = self.tokenizer.batch_encode_plus(sequence,
-                                                max_length=100,
-                                                pad_to_max_length=True)
+            sequence = X[self.text_input_column].values.tolist()
+        seqs = self.tokenizer.batch_encode_plus(
+            sequence, max_length=self.seq_size, padding="max_length", truncation=True
+        )
 
-        return np.asarray(seqs['input_ids']), np.asarray(seqs['attention_mask'])
+        return (
+            np.asarray(seqs["input_ids"]),
+            np.asarray(seqs["attention_mask"]),
+        )
+
+    def _prepare_data(self, X, y, validation_data=None):
+        """Prepares the data for training and validation.
+        1- Encodes y to categorical
+        2- Differentiates data preparation for bert models and non-bert models
+        3- Creates embedding matrix and init model for training data only (validation_data=None)
+
+        Parameters
+        ----------
+        X : pd.DataFrame
+
+        y : pd.Series
+
+        validation_data : None or tuple
+            Tuple of validation data
+            Data on which to evaluate the loss and any model metrics at the end of each epoch. The model will not be
+            trained on this data. This could be a tuple (x_val, y_val). validation_data will override validation_split.
+            Default value, None.
+
+        Returns
+        -------
+        X_input : pd.DataFrame
+
+        y_categorical : pd.Series
+        """
+        y_categorical = to_categorical(y)
+        if not validation_data:
+            nb_labels = len(np.unique(y))
+
+        if self.architecture_function.__name__ != "bert_model":
+            X = self.tokenizer.transform(X)
+            X_meta, nb_meta_features = self._get_meta(X)
+
+            if not validation_data:
+                if self.pretrained_embedding:
+                    self._get_embedding_matrix()
+                else:
+                    self._create_vocabulary_from_tokens(X)
+                    self._generate_random_embedding_matrix()
+                self.vocabulary_dict = {
+                    word: i for i, word in enumerate(self.vocabulary)
+                }
+                self.model = self.architecture_function(
+                    embedding_matrix_init=self.embedding_matrix,
+                    ntargets=nb_labels,
+                    seq_max=self.seq_size,
+                    nb_meta=nb_meta_features,
+                    loss=self.loss,
+                    activation=self.activation,
+                )
+
+            X_seq = self._prepare_sequences(X)
+
+            if nb_meta_features == 0:
+                X_input = X_seq
+            else:
+                X_input = [X_seq, X_meta]
+
+        else:
+            X_seq, X_attention = self._prepare_bert_sequences(X)
+            X_meta, nb_meta_features = self._get_meta(X)
+            self.nb_labels, self.nb_meta_features = nb_labels, nb_meta_features
+            if not validation_data:
+                self.model = self.architecture_function(
+                    ntargets=nb_labels,
+                    seq_max=self.seq_size,
+                    nb_meta=nb_meta_features,
+                    loss=self.loss,
+                    activation=self.activation,
+                    bert_model=self.bert_model,
+                )
+
+            if nb_meta_features == 0:
+                X_input = [X_seq, X_attention]
+            else:
+                X_input = [X_seq, X_attention, X_meta]
+
+        return X_input, y_categorical
 
     def _get_meta(self, X):
         """Returns as a pd.DataFrame the metadata from X given the list_meta
@@ -405,18 +544,23 @@ class NeuralModel(BaseEstimator, ClassifierMixin):
             nb_meta_features = 0
         else:
             meta_input_list = self.meta_input_list
-            meta_input_list = [col + '__' for col in meta_input_list]
+            meta_input_list = [col + "__" for col in meta_input_list]
 
             if isinstance(X, dict):
                 columns_list = list(X.keys())
             else:
                 columns_list = list(X.columns)
 
-            meta_columns_list = [col for col in columns_list
-                                 if col.startswith(tuple(meta_input_list))]
+            meta_columns_list = [
+                col for col in columns_list if col.startswith(tuple(meta_input_list))
+            ]
 
             if isinstance(X, dict):
-                X_meta = np.array([[X[meta_feature] for meta_feature in meta_columns_list], ])
+                X_meta = np.array(
+                    [
+                        [X[meta_feature] for meta_feature in meta_columns_list],
+                    ]
+                )
             else:
                 X_meta = X[meta_columns_list]
 
