@@ -4,14 +4,17 @@ from gensim.models.keyedvectors import Vocab, KeyedVectors
 from sklearn.feature_extraction.text import TfidfVectorizer
 from sklearn.decomposition import TruncatedSVD
 from sklearn.feature_extraction.text import CountVectorizer
+
 # from glove import Corpus, Glove
 
 from melusine.utils.streamer import Streamer
 
 log = logging.getLogger(__name__)
 log.setLevel(logging.DEBUG)
-formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s',
-                              datefmt='%d/%m %I:%M')
+formatter = logging.Formatter(
+    "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
+    datefmt="%d/%m %I:%M",
+)
 
 
 class Embedding:
@@ -46,17 +49,18 @@ class Embedding:
     >>> embedding = Embedding().load(filepath)
     """
 
-    def __init__(self,
-                 input_column=None,
-                 tokens_column=None,
-                 workers=40,
-                 random_seed=42,
-                 iter=15,
-                 size=300,
-                 method="word2vec_cbow",
-                 stop_removal=True,
-                 min_count=100
-                 ):
+    def __init__(
+        self,
+        input_column=None,
+        tokens_column=None,
+        workers=40,
+        random_seed=42,
+        iter=15,
+        size=300,
+        method="word2vec_cbow",
+        stop_removal=True,
+        min_count=100,
+    ):
         """
         Parameters :
         ----------
@@ -92,7 +96,7 @@ class Embedding:
         ch.setLevel(logging.INFO)
         ch.setFormatter(formatter)
         self.logger.addHandler(ch)
-        self.logger.debug('Create an Embedding instance.')
+        self.logger.debug("Create an Embedding instance.")
         self.input_column = input_column
         self.tokens_column = tokens_column
         self.input_data = None
@@ -102,27 +106,29 @@ class Embedding:
         self.method = method
         self.workers = workers
 
-        if self.method in ['word2vec_sg', 'word2vec_cbow']:
-            self.train_params = {"size": size,
-                                 "alpha": 0.025,
-                                 "min_count": min_count,
-                                 "max_vocab_size": None,
-                                 "sample": 0.001,
-                                 "seed": random_seed,
-                                 "workers": workers,
-                                 "min_alpha": 0.0001,
-                                 "negative": 5,
-                                 "hs": 0,
-                                 "ns_exponent": 0.75,
-                                 "cbow_mean": 1,
-                                 "iter": iter,
-                                 "null_word": 0,
-                                 "trim_rule": None,
-                                 "sorted_vocab": 1,
-                                 "batch_words": 10000,
-                                 "compute_loss": False,
-                                 "callbacks": (),
-                                 "max_final_vocab": None}
+        if self.method in ["word2vec_sg", "word2vec_cbow"]:
+            self.train_params = {
+                "size": size,
+                "alpha": 0.025,
+                "min_count": min_count,
+                "max_vocab_size": None,
+                "sample": 0.001,
+                "seed": random_seed,
+                "workers": workers,
+                "min_alpha": 0.0001,
+                "negative": 5,
+                "hs": 0,
+                "ns_exponent": 0.75,
+                "cbow_mean": 1,
+                "iter": iter,
+                "null_word": 0,
+                "trim_rule": None,
+                "sorted_vocab": 1,
+                "batch_words": 10000,
+                "compute_loss": False,
+                "callbacks": (),
+                "max_final_vocab": None,
+            }
 
             if self.method == "word2vec_sg":
                 self.train_params["sg"] = 1
@@ -132,14 +138,14 @@ class Embedding:
                 self.train_params["sg"] = 0
                 self.train_params["window"] = 5
 
-        elif self.method in 'lsa_tfidf':
+        elif self.method in "lsa_tfidf":
             self.train_params = {
                 # TfidfVectorizer Parameters
-                "vectorizer_encoding": 'utf-8',
-                "vectorizer_decode_error": 'strict',
+                "vectorizer_encoding": "utf-8",
+                "vectorizer_decode_error": "strict",
                 "vectorizer_strip_accents": None,
                 "vectorizer_lowercase": False,
-                "vectorizer_analyzer": 'word',
+                "vectorizer_analyzer": "word",
                 "vectorizer_stop_words": None,
                 "vectorizer_ngram_range": (1, 1),
                 "vectorizer_max_df": 1.0,
@@ -147,41 +153,44 @@ class Embedding:
                 "vectorizer_max_features": None,
                 "vectorizer_vocabulary": None,
                 "vectorizer_binary": False,
-                "vectorizer_norm": 'l2',
+                "vectorizer_norm": "l2",
                 "vectorizer_use_idf": True,
                 "vectorizer_smooth_idf": True,
                 "vectorizer_sublinear_tf": False,
-
                 # TruncatedSVD Parameters
                 "svd_n_components": size,
-                "svd_algorithm": 'randomized',
+                "svd_algorithm": "randomized",
                 "svd_n_iter": iter,
                 "svd_random_state": random_seed,
-                "svd_tol": 0.0
+                "svd_tol": 0.0,
             }
-        elif self.method == 'lsa_docterm':
+        elif self.method == "lsa_docterm":
             # CountVectorizer Parameters
             # TruncatedSVD Parameters
-            self.train_params = {"vectorizer_encoding": 'utf-8',
-                                 "vectorizer_decode_error": 'strict',
-                                 "vectorizer_strip_accents": None,
-                                 "vectorizer_lowercase": False,
-                                 "vectorizer_stop_words": None,
-                                 "vectorizer_ngram_range": (1, 1),
-                                 "vectorizer_analyzer": 'word',
-                                 "vectorizer_max_df": 1.0,
-                                 "vectorizer_min_df": 1,
-                                 "vectorizer_max_features": None,
-                                 "vectorizer_vocabulary": None,
-                                 "vectorizer_binary": False,
-                                 "svd_n_components": size,
-                                 "svd_algorithm": 'randomized',
-                                 "svd_n_iter": iter,
-                                 "svd_random_state": random_seed,
-                                 "svd_tol": 0.0}
+            self.train_params = {
+                "vectorizer_encoding": "utf-8",
+                "vectorizer_decode_error": "strict",
+                "vectorizer_strip_accents": None,
+                "vectorizer_lowercase": False,
+                "vectorizer_stop_words": None,
+                "vectorizer_ngram_range": (1, 1),
+                "vectorizer_analyzer": "word",
+                "vectorizer_max_df": 1.0,
+                "vectorizer_min_df": 1,
+                "vectorizer_max_features": None,
+                "vectorizer_vocabulary": None,
+                "vectorizer_binary": False,
+                "svd_n_components": size,
+                "svd_algorithm": "randomized",
+                "svd_n_iter": iter,
+                "svd_random_state": random_seed,
+                "svd_tol": 0.0,
+            }
 
         else:
-            raise ValueError(f"Error: Embedding method {method} not recognized or not implemented yet ;).")
+            raise ValueError(
+                f"Error: Embedding method {method} not recognized or not implemented yet ;)."
+            )
 
     def save(self, filepath):
         """Method to save Embedding object."""
@@ -189,7 +198,7 @@ class Embedding:
 
     def load(self, filepath):
         """Method to load Embedding object."""
-        self.embedding = KeyedVectors.load(filepath, mmap='r')
+        self.embedding = KeyedVectors.load(filepath, mmap="r")
         return self
 
     def train(self, X):
@@ -200,13 +209,15 @@ class Embedding:
             Containing a clean body column.
         """
 
-        self.logger.info('Start training for embedding')
+        self.logger.info("Start training for embedding")
 
         if (not self.input_column) and (not self.tokens_column):
-            raise ValueError("""
-            Please specify one of the following keyword argument when instanciating an Embedding class object: 
-            - input_column argument (containing raw text) 
-            - tokens_column argument (containing) list of tokens""")
+            raise ValueError(
+                """
+            Please specify one of the following keyword argument when instanciating an Embedding class object:
+            - input_column argument (containing raw text)
+            - tokens_column argument (containing) list of tokens"""
+            )
 
         # If tokens column is provided: use it. Otherwise, use input column
         if self.tokens_column:
@@ -215,21 +226,23 @@ class Embedding:
             self.streamer.to_stream(X)
             self.input_data = self.streamer.stream
         else:
-            raise ValueError("""
-            Please specify one of the following keyword argument when instanciating an Embedding class object: 
-            - input_column argument (containing raw text) 
-            - tokens_column argument (containing) list of tokens""")
+            raise ValueError(
+                """
+            Please specify one of the following keyword argument when instanciating an Embedding class object:
+            - input_column argument (containing raw text)
+            - tokens_column argument (containing) list of tokens"""
+            )
 
-        if self.method in ['word2vec_sg', 'word2vec_cbow']:
+        if self.method in ["word2vec_sg", "word2vec_cbow"]:
             self.train_word2vec()
-        elif self.method == 'lsa_tfidf':
+        elif self.method == "lsa_tfidf":
             self.train_tfidf()
-        elif self.method == 'lsa_docterm':
+        elif self.method == "lsa_docterm":
             self.train_docterm()
         # elif embedding_type == 'glove':
         #    self.train_glove(X, self.input_column)
 
-        self.logger.info('Done.')
+        self.logger.info("Done.")
 
     def train_tfidf(self):
         """Train a TF-IDF Vectorizer to compute a TF-IDFized Doc-Term Matrix relative to a corpus.
@@ -245,7 +258,6 @@ class Embedding:
             tokenizer=dummy_function,
             preprocessor=dummy_function,
             token_pattern=None,
-
             encoding=self.train_params["vectorizer_encoding"],
             decode_error=self.train_params["vectorizer_decode_error"],
             strip_accents=self.train_params["vectorizer_strip_accents"],
@@ -260,7 +272,7 @@ class Embedding:
             binary=self.train_params["vectorizer_binary"],
             use_idf=self.train_params["vectorizer_use_idf"],
             smooth_idf=self.train_params["vectorizer_smooth_idf"],
-            sublinear_tf=self.train_params["vectorizer_sublinear_tf"]
+            sublinear_tf=self.train_params["vectorizer_sublinear_tf"],
         )
 
         tfidf_data = tfidf_vec.fit_transform(self.input_data)
@@ -288,7 +300,6 @@ class Embedding:
             tokenizer=dummy_function,
             preprocessor=dummy_function,
             token_pattern=None,
-
             encoding=self.train_params["vectorizer_encoding"],
             decode_error=self.train_params["vectorizer_decode_error"],
             strip_accents=self.train_params["vectorizer_strip_accents"],
@@ -300,7 +311,7 @@ class Embedding:
             min_df=self.train_params["vectorizer_min_df"],
             max_features=self.train_params["vectorizer_max_features"],
             vocabulary=self.train_params["vectorizer_vocabulary"],
-            binary=self.train_params["vectorizer_binary"]
+            binary=self.train_params["vectorizer_binary"],
         )
 
         docterm_data = count_vec.fit_transform(self.input_data)
@@ -318,48 +329,53 @@ class Embedding:
             Sklearn object on which the TruncatedSVD will be computed.
         """
 
-        svd = TruncatedSVD(n_components=self.train_params["svd_n_components"],
-                           algorithm=self.train_params["svd_algorithm"],
-                           n_iter=self.train_params["svd_n_iter"],
-                           random_state=self.train_params["svd_random_state"],
-                           tol=self.train_params["svd_tol"])
+        svd = TruncatedSVD(
+            n_components=self.train_params["svd_n_components"],
+            algorithm=self.train_params["svd_algorithm"],
+            n_iter=self.train_params["svd_n_iter"],
+            random_state=self.train_params["svd_random_state"],
+            tol=self.train_params["svd_tol"],
+        )
         svd.fit(vectorized_corpus_data)
 
         embedding_matrix = svd.components_.T
         return embedding_matrix
 
     def train_word2vec(self):
-        """Fits a Word2Vec Embedding on the given documents, and update the embedding attribute.
-        """
+        """Fits a Word2Vec Embedding on the given documents, and update the embedding attribute."""
 
-        embedding = Word2Vec(size=self.train_params["size"],
-                             alpha=self.train_params["alpha"],
-                             window=self.train_params["window"],
-                             min_count=self.train_params["min_count"],
-                             max_vocab_size=self.train_params["max_vocab_size"],
-                             sample=self.train_params["sample"],
-                             seed=self.train_params["seed"],
-                             workers=self.train_params["workers"],
-                             min_alpha=self.train_params["min_alpha"],
-                             sg=self.train_params["sg"],
-                             hs=self.train_params["hs"],
-                             negative=self.train_params["negative"],
-                             ns_exponent=self.train_params["ns_exponent"],
-                             cbow_mean=self.train_params["cbow_mean"],
-                             iter=self.train_params["iter"],
-                             null_word=self.train_params["null_word"],
-                             trim_rule=self.train_params["trim_rule"],
-                             sorted_vocab=self.train_params["sorted_vocab"],
-                             batch_words=self.train_params["batch_words"],
-                             compute_loss=self.train_params["compute_loss"],
-                             callbacks=self.train_params["callbacks"],
-                             max_final_vocab=self.train_params["max_final_vocab"])
+        embedding = Word2Vec(
+            size=self.train_params["size"],
+            alpha=self.train_params["alpha"],
+            window=self.train_params["window"],
+            min_count=self.train_params["min_count"],
+            max_vocab_size=self.train_params["max_vocab_size"],
+            sample=self.train_params["sample"],
+            seed=self.train_params["seed"],
+            workers=self.train_params["workers"],
+            min_alpha=self.train_params["min_alpha"],
+            sg=self.train_params["sg"],
+            hs=self.train_params["hs"],
+            negative=self.train_params["negative"],
+            ns_exponent=self.train_params["ns_exponent"],
+            cbow_mean=self.train_params["cbow_mean"],
+            iter=self.train_params["iter"],
+            null_word=self.train_params["null_word"],
+            trim_rule=self.train_params["trim_rule"],
+            sorted_vocab=self.train_params["sorted_vocab"],
+            batch_words=self.train_params["batch_words"],
+            compute_loss=self.train_params["compute_loss"],
+            callbacks=self.train_params["callbacks"],
+            max_final_vocab=self.train_params["max_final_vocab"],
+        )
 
         # TODO Fix Streamer
         embedding.build_vocab(self.input_data)
-        embedding.train(self.input_data,
-                        total_examples=embedding.corpus_count,
-                        epochs=self.train_params["iter"])
+        embedding.train(
+            self.input_data,
+            total_examples=embedding.corpus_count,
+            epochs=self.train_params["iter"],
+        )
 
         self.embedding = embedding.wv
 
@@ -389,9 +405,10 @@ class Embedding:
             Word vocabulary (key: word, value: word_index)
         """
 
-        vocab = {word: word2id[word] for word in sorted(word2id,
-                                                        key=word2id.__getitem__,
-                                                        reverse=False)}
+        vocab = {
+            word: word2id[word]
+            for word in sorted(word2id, key=word2id.__getitem__, reverse=False)
+        }
         embedding_matrix = embedding_matrix
         vector_size = embedding_matrix.shape[1]
 
@@ -401,6 +418,8 @@ class Embedding:
 
         kv.index2word = list(vocab.keys())
 
-        kv.vocab = {word: Vocab(index=word_id, count=0) for word, word_id in vocab.items()}
+        kv.vocab = {
+            word: Vocab(index=word_id, count=0) for word, word_id in vocab.items()
+        }
 
         self.embedding = kv
